module mat_cmplx
  use la_precision, only: WP=>DP
  use f95_lapack,   only: GEEV => LA_GEEV, HEEV => LA_HEEV
  use mat_base
  implicit none

  type, extends(matrice) :: matrice_cmplx
    private
    complex(kind=WP), dimension(:,:), allocatable :: mat
    complex(kind=WP), dimension(:),   allocatable :: valp
    character(len=100) :: format_sortie_mat = &
                   '(*(:,"(",f6.3,",",f6.3,")",1x))'
    character(len=100) :: format_sortie_valp = &
                   '("Valeur propre N.",i2," : (",es10.3,"  , ",es10.3,")")'
  contains
    procedure :: propres => propres_cmplx
    procedure :: sorties => sorties_cmplx
  end type matrice_cmplx

  type, extends(matrice_cmplx) :: matrice_cmplx_vecp
    private
    complex(kind=WP), dimension(:,:), allocatable :: vecp
    character(len=100) :: format_sortie_vecp = &
                   '(*(:,"(",f6.3,",",f6.3,")",1x))'
  contains
    procedure :: propres => propres_cmplx_vecp
    procedure :: sorties => sorties_cmplx_vecp
  end type matrice_cmplx_vecp

  type, extends(matrice_cmplx) :: matrice_hermitienne
  contains
    procedure :: propres => propres_hermitienne
  end type matrice_hermitienne

  type, extends(matrice_hermitienne) :: matrice_hermitienne_vecp
    private
    complex(kind=WP), dimension(:,:), allocatable :: vecp
    character(len=100) :: format_sortie_vecp = &
                   '(*(:,"(",f6.3,",",f6.3,")",1x))'
  contains
    procedure :: propres => propres_hermitienne_vecp
    procedure :: sorties => sorties_hermitienne_vecp
  end type matrice_hermitienne_vecp

  interface matrice_cmplx
    module procedure matrice_cmplx_f
  end interface matrice_cmplx

  interface matrice_cmplx_vecp
    module procedure matrice_cmplx_vecp_f
  end interface matrice_cmplx_vecp

  interface matrice_hermitienne
    module procedure matrice_hermitienne_f
  end interface matrice_hermitienne

  interface matrice_hermitienne_vecp
    module procedure matrice_hermitienne_vecp_f
  end interface matrice_hermitienne_vecp

contains

  function matrice_cmplx_f(fichier)
    character(len=*), intent(in) :: fichier
    character(len=30)            :: type_matrice
    type(matrice_cmplx)          :: matrice_cmplx_f
    integer n, i, j, unite

    associate(t => matrice_cmplx_f)
      open( newunit=unite,         &
            file=fichier,          &
            action="read",         &
            form="formatted",      &
            access="sequential" )
      read(unit=unite, fmt='(a, /, i4)') type_matrice, n
      t%matrice%ordre = n
      t%type = type_matrice
      allocate(t%mat(n, n))
      allocate(t%valp(n))
      read(unit=unite, fmt=*) ((t%mat(i,j),j=1,n),i=1,n)
      close(unit=unite)
      call matrice_cmplx_f%sorties(init=.true.)
    end associate
  end function matrice_cmplx_f

  function matrice_cmplx_vecp_f(fichier)
    character(len=*), intent(in) :: fichier
    type(matrice_cmplx_vecp)     :: matrice_cmplx_vecp_f
    integer n

    matrice_cmplx_vecp_f%matrice_cmplx = matrice_cmplx(fichier)
    n = matrice_cmplx_vecp_f%get_ordre()
    allocate(matrice_cmplx_vecp_f%vecp(n,n))
  end function matrice_cmplx_vecp_f

  function matrice_hermitienne_f(fichier)
    character(len=*), intent(in) :: fichier
    type(matrice_hermitienne)    :: matrice_hermitienne_f

    matrice_hermitienne_f%matrice_cmplx = matrice_cmplx(fichier)
  end function matrice_hermitienne_f

  function matrice_hermitienne_vecp_f(fichier)
    character(len=*), intent(in)   :: fichier
    type(matrice_hermitienne_vecp) :: matrice_hermitienne_vecp_f
    integer n

    matrice_hermitienne_vecp_f%matrice_hermitienne = matrice_hermitienne(fichier)
    n = matrice_hermitienne_vecp_f%get_ordre()
    allocate(matrice_hermitienne_vecp_f%vecp(n,n))
  end function matrice_hermitienne_vecp_f

  subroutine propres_cmplx(this)
    class(matrice_cmplx), intent(inout) :: this

    call geev(this%mat, this%valp)
  end subroutine propres_cmplx

  subroutine propres_cmplx_vecp(this)
    class(matrice_cmplx_vecp), intent(inout) :: this

    call geev(this%mat, this%valp, VR=this%vecp)
  end subroutine propres_cmplx_vecp

  subroutine propres_hermitienne(this)
    class(matrice_hermitienne), intent(inout)  :: this
    REAL(WP), DIMENSION(:), ALLOCATABLE :: W
    integer n

    n = this%get_ordre()
    allocate(W(n))
    call heev(this%mat, w)
    this%valp(:) = W(:)
    deallocate(W)
  end subroutine propres_hermitienne

  subroutine propres_hermitienne_vecp(this)
    class(matrice_hermitienne_vecp), intent(inout) :: this
    REAL(WP), DIMENSION(:), ALLOCATABLE :: W
    integer n

    n = this%get_ordre()
    allocate(W(n))
    call heev(this%mat, w, JOBZ='V')
    this%valp(:) = W(:)
    this%vecp(:,:) = this%mat(:,:)
    deallocate(W)
  end subroutine propres_hermitienne_vecp

  subroutine sorties_cmplx(this, init)
    class(matrice_cmplx), intent(in) :: this
    logical, optional, intent(in)    :: init
    logical init_loc
    integer i

    if (     present(init)) init_loc = init
    if (.NOT.present(init)) init_loc = .false.

    if (init_loc) then
      print '(//, t20, a, /)', this%type
      do i=1,size(this%mat,1)
        print trim(this%format_sortie_mat), this%mat(i,:)
      end do
    else
      print '(//, t20, "Valeurs propres", /)'
      do i=1,size(this%valp)
        print trim(this%format_sortie_valp), i, this%valp(i)
      end do
    end if
  end subroutine sorties_cmplx

  subroutine sorties_cmplx_vecp(this, init)
    class(matrice_cmplx_vecp), intent(in) :: this
    logical, optional, intent(in)         :: init
    integer i
 
    call this%matrice_cmplx%sorties
    print '(//, t20, a, /)', "Vecteurs propres"
    do i=1,size(this%vecp,1)
      print this%format_sortie_vecp, this%vecp(i,:)
    end do
  end subroutine sorties_cmplx_vecp

  subroutine sorties_hermitienne_vecp(this, init)
    class(matrice_hermitienne_vecp), intent(in) :: this
    logical, optional, intent(in)               :: init
    integer i

    call this%matrice_cmplx%sorties
    print '(//, t20, a, /)', "Vecteurs propres"
    do i=1,size(this%vecp,1)
      print trim(this%format_sortie_vecp), this%vecp(i,:)
    end do
  end subroutine sorties_hermitienne_vecp

end module mat_cmplx
