submodule(mat_real) smat_real
  use la_precision, only: WP=>DP
  use f95_lapack,   only: GEEV => LA_GEEV, SYEV => LA_SYEV
  implicit none

contains

  module function matrice_reelle_f(fichier)
    character(len=*), intent(in) :: fichier
    character(len=30)            :: type_matrice
    type(matrice_reelle)         :: matrice_reelle_f
    integer n, i, j, unite

    open( newunit=unite,         &
          file=fichier,          &
          action="read",         &
          form="formatted",      &
          access="sequential" )
    read(unit=unite, fmt='(a,/,i4)') type_matrice, n
    matrice_reelle_f%matrice%ordre = n
    matrice_reelle_f%type = type_matrice
    allocate(matrice_reelle_f%mat(n, n))
    allocate(matrice_reelle_f%valp(n))
    read(unit=unite, fmt=*) ((matrice_reelle_f%mat(i,j),j=1,n),i=1,n)
    close(unit=unite)
    call matrice_reelle_f%sorties(init=.true.)
  end function matrice_reelle_f

  module function matrice_reelle_vecp_f(fichier)
    character(len=*), intent(in) :: fichier
    type(matrice_reelle_vecp)    :: matrice_reelle_vecp_f
    integer n

    matrice_reelle_vecp_f%matrice_reelle = matrice_reelle(fichier)
    n = matrice_reelle_vecp_f%get_ordre()
    allocate(matrice_reelle_vecp_f%vecp(n,n))
  end function matrice_reelle_vecp_f

  module function matrice_reelle_sym_f(fichier)
    character(len=*), intent(in) :: fichier
    type(matrice_reelle_sym)     :: matrice_reelle_sym_f

    matrice_reelle_sym_f%matrice_reelle = matrice_reelle(fichier)
  end function matrice_reelle_sym_f

  module function matrice_reelle_sym_vecp_f(fichier)
    character(len=*), intent(in)  :: fichier
    type(matrice_reelle_sym_vecp) :: matrice_reelle_sym_vecp_f
    integer n

    matrice_reelle_sym_vecp_f%matrice_reelle_sym = matrice_reelle_sym(fichier)
    n = matrice_reelle_sym_vecp_f%get_ordre()
    allocate(matrice_reelle_sym_vecp_f%vecp(n,n))
  end function matrice_reelle_sym_vecp_f

  module subroutine propres_reelle(this)
    class(matrice_reelle), intent(inout) :: this
    REAL(WP), DIMENSION(:), ALLOCATABLE :: WR, WI
    integer n

    n = this%get_ordre()
    allocate(wr(n), wi(n))
    call geev(this%mat, WR, WI)
    this%valp(:) = cmplx(WR(:), WI(:), kind=WP)
    deallocate(WR, WI)
  end subroutine propres_reelle

  module subroutine propres_reelle_vecp(this)
    class(matrice_reelle_vecp), intent(inout) :: this
    REAL(WP), ALLOCATABLE :: VR(:,:)
    REAL(WP), DIMENSION(:), ALLOCATABLE :: WR, WI
    integer n

    n = this%get_ordre()
    allocate(WR(n), WI(n), vr(n,n))
    call geev(this%mat, WR, WI, VR=vr)
    this%valp(:) = cmplx(WR(:), WI(:), kind=WP)
    deallocate(WR, WI)
    this%vecp(:,:) = recup_vect_propres()
    deallocate(vr)
    contains
      function recup_vect_propres()
        complex(kind=WP), dimension(n,n) :: recup_vect_propres
        logical saut_indice
        integer i
  
        saut_indice = .false.
        do i=1,n
          if (AIMAG(this%valp(i)) == 0.) then
            recup_vect_propres(:,i) = vr(:,i)
          else if (saut_indice) then
            saut_indice = .false.
            cycle
          else
            recup_vect_propres(:,i)   = &
                           cmplx(vr(:,i),  vr(:,i+1), kind=WP)
            recup_vect_propres(:,i+1) = &
                           cmplx(vr(:,i), -vr(:,i+1), kind=WP)
            saut_indice = .true.
          end if
        end do
      end function recup_vect_propres
  end subroutine propres_reelle_vecp

  module subroutine propres_reelle_sym(this)
    class(matrice_reelle_sym), intent(inout) :: this
    REAL(WP), ALLOCATABLE :: w(:)
    integer n

    n = this%get_ordre()
    allocate(w(n))
    call syev(this%mat, w, UPLO='L')
    this%valp(:) = w(:)
    deallocate(w)
  end subroutine propres_reelle_sym

  module subroutine propres_reelle_sym_vecp(this)
    class(matrice_reelle_sym_vecp), intent(inout) :: this
    REAL(WP), ALLOCATABLE :: w(:)
    integer n

    n = this%get_ordre()
    allocate(w(n))
    call syev(this%mat, w , JOBZ='V', UPLO='L')
    this%valp(:) = w(:)
    this%vecp(:,:) = this%mat(:,:)
  end subroutine propres_reelle_sym_vecp

  module subroutine sorties_reelle(this, init)
    class(matrice_reelle), intent(in) :: this
    logical, optional, intent(in)     :: init
    integer i
    logical init_loc

    if (     present(init)) init_loc = init
    if (.NOT.present(init)) init_loc = .false.

    if (init_loc) then
      print '(//, t20, a, /)', this%type
      do i=1,size(this%mat,1)
        print trim(this%format_sortie_mat), this%mat(i,:)
      end do
    else
      print '(//, t20, "Valeurs propres", /)'
      do i=1,size(this%valp)
        print trim(this%format_sortie_valp), i, this%valp(i)
      end do
    end if
  end subroutine sorties_reelle

  module subroutine sorties_reelle_vecp(this, init)
    class(matrice_reelle_vecp), intent(in) :: this
    logical, optional, intent(in)          :: init
    integer i

    call this%matrice_reelle%sorties
    print '(//, t20, a, /)', "Vecteurs propres"
    do i=1,size(this%vecp,1)
      print this%format_sortie_vecp, this%vecp(i,:)
    end do
  end subroutine sorties_reelle_vecp

  module subroutine sorties_reelle_sym_vecp(this, init)
    class(matrice_reelle_sym_vecp), intent(in) :: this
    logical, optional, intent(in)              :: init
    integer i

    call this%matrice_reelle%sorties
    print '(//, t20, a, /)', "Vecteurs propres"
    do i=1,size(this%vecp,1)
      print trim(this%format_sortie_vecp), this%vecp(i,:)
    end do
  end subroutine sorties_reelle_sym_vecp
end submodule smat_real
