MODULE F77_LAPACK
  INTERFACE LA_ILAENV
    FUNCTION ILAENV( ISPEC, NAME, OPTS, N1, N2, N3, N4 )
      INTEGER :: ILAENV
      CHARACTER(LEN=*), INTENT(IN) :: NAME, OPTS
      INTEGER, INTENT(IN) :: ISPEC, N1, N2, N3, N4
    END FUNCTION ILAENV
  END INTERFACE

  INTERFACE LA_SYEV
    SUBROUTINE SSYEV( JOBZ, UPLO, N, A, LDA, W, WORK, LWORK, INFO )
      USE LA_PRECISION, ONLY: WP => SP
      CHARACTER(LEN=1), INTENT(IN) :: JOBZ, UPLO
      INTEGER, INTENT(IN) :: LDA, LWORK, N
      INTEGER, INTENT(OUT) :: INFO
      REAL(WP), INTENT(INOUT) :: A(LDA,*)
      REAL(WP), INTENT(OUT) :: W(*)
      REAL(WP), INTENT(OUT) :: WORK(*)
    END SUBROUTINE SSYEV

    SUBROUTINE DSYEV( JOBZ, UPLO, N, A, LDA, W, WORK, LWORK, INFO )
      USE LA_PRECISION, ONLY: WP => DP
      CHARACTER(LEN=1), INTENT(IN) :: JOBZ, UPLO
      INTEGER, INTENT(IN) :: LDA, LWORK, N
      INTEGER, INTENT(OUT) :: INFO
      REAL(WP), INTENT(INOUT) :: A(LDA,*)
      REAL(WP), INTENT(OUT) :: W(*)
      REAL(WP), INTENT(OUT) :: WORK(*)
    END SUBROUTINE DSYEV
  END INTERFACE

  INTERFACE LA_HEEV
    SUBROUTINE CHEEV( JOBZ, UPLO, N, A, LDA, W, WORK, LWORK, RWORK,  &
   &                  INFO )
      USE LA_PRECISION, ONLY: WP => SP
      CHARACTER(LEN=1), INTENT(IN) :: JOBZ, UPLO
      INTEGER, INTENT(IN) :: LDA, LWORK, N
      INTEGER, INTENT(OUT) :: INFO
      COMPLEX(WP), INTENT(INOUT) :: A(LDA,*)
      REAL(WP), INTENT(OUT) :: W(*), RWORK(*)
      COMPLEX(WP), INTENT(OUT) :: WORK(*)
    END SUBROUTINE CHEEV

    SUBROUTINE ZHEEV( JOBZ, UPLO, N, A, LDA, W, WORK, LWORK, RWORK,  &
   &                  INFO )
      USE LA_PRECISION, ONLY: WP => DP
      CHARACTER(LEN=1), INTENT(IN) :: JOBZ, UPLO
      INTEGER, INTENT(IN) :: LDA, LWORK, N
      INTEGER, INTENT(OUT) :: INFO
      COMPLEX(WP), INTENT(INOUT) :: A(LDA,*)
      REAL(WP), INTENT(OUT) :: W(*), RWORK(*)
      COMPLEX(WP), INTENT(OUT) :: WORK(*)
    END SUBROUTINE ZHEEV
  END INTERFACE

  INTERFACE LA_GEEV
    SUBROUTINE SGEEV( JOBVL, JOBVR, N, A, LDA, WR, WI, VL, LDVL, VR, &
   &                  LDVR, WORK, LWORK, INFO )
      USE LA_PRECISION, ONLY: WP => SP
      CHARACTER(LEN=1), INTENT(IN) :: JOBVL, JOBVR
      INTEGER, INTENT(IN) :: N, LDA, LDVL, LDVR, LWORK
      INTEGER, INTENT(OUT) :: INFO
      REAL(WP), INTENT(INOUT) :: A(LDA,*)
      REAL(WP), INTENT(OUT) :: VL(LDVL,*), VR(LDVR,*), WR(*), WI(*), &
   &                           WORK(*)
    END SUBROUTINE SGEEV

    SUBROUTINE DGEEV( JOBVL, JOBVR, N, A, LDA, WR, WI, VL, LDVL, VR, &
   &                  LDVR, WORK, LWORK, INFO )
      USE LA_PRECISION, ONLY: WP => DP
      CHARACTER(LEN=1), INTENT(IN) :: JOBVL, JOBVR
      INTEGER, INTENT(IN) :: N, LDA, LDVL, LDVR, LWORK
      INTEGER, INTENT(OUT) :: INFO
      REAL(WP), INTENT(INOUT) :: A(LDA,*)
      REAL(WP), INTENT(OUT) :: VL(LDVL,*), VR(LDVR,*), WR(*), WI(*), &
   &                           WORK(*)
    END SUBROUTINE DGEEV

    SUBROUTINE CGEEV( JOBVL, JOBVR, N, A, LDA, W, VL, LDVL, VR, LDVR,&
   &                  WORK, LWORK, RWORK, INFO )
      USE LA_PRECISION, ONLY: WP => SP
      CHARACTER(LEN=1), INTENT(IN) :: JOBVL, JOBVR
      INTEGER, INTENT(IN) :: N, LDA, LDVL, LDVR, LWORK
      INTEGER, INTENT(OUT) :: INFO
      REAL(WP), INTENT(OUT) :: RWORK(*)
      COMPLEX(WP), INTENT(INOUT) :: A(LDA,*)
      COMPLEX(WP), INTENT(OUT) :: VL(LDVL,*), VR(LDVR,*), W(*),      &
   &                              WORK(*)
    END SUBROUTINE CGEEV

    SUBROUTINE ZGEEV( JOBVL, JOBVR, N, A, LDA, W, VL, LDVL, VR, LDVR,&
   &                  WORK, LWORK, RWORK, INFO )
      USE LA_PRECISION, ONLY: WP => DP
      CHARACTER(LEN=1), INTENT(IN) :: JOBVL, JOBVR
      INTEGER, INTENT(IN) :: N, LDA, LDVL, LDVR, LWORK
      INTEGER, INTENT(OUT) :: INFO
      REAL(WP), INTENT(OUT) :: RWORK(*)
      COMPLEX(WP), INTENT(INOUT) :: A(LDA,*)
      COMPLEX(WP), INTENT(OUT) :: VL(LDVL,*), VR(LDVR,*), W(*),      &
   &                              WORK(*)
    END SUBROUTINE ZGEEV
  END INTERFACE
END MODULE F77_LAPACK
